@@ -1,10 +1,19 @@
 package com.actelion.research.chem.phesaflex;
 
 
-import java.util.Arrays;
 import java.util.Map;
 
+import com.actelion.research.chem.Coordinates;
 import com.actelion.research.chem.StereoMolecule;
+import com.actelion.research.chem.alignment3d.transformation.ExponentialMap;
+import com.actelion.research.chem.alignment3d.transformation.Quaternion;
+import com.actelion.research.chem.alignment3d.transformation.Rotation;
+import com.actelion.research.chem.alignment3d.transformation.RotationDerivatives;
+import com.actelion.research.chem.alignment3d.transformation.TransformationSequence;
+import com.actelion.research.chem.alignment3d.transformation.Translation;
+import com.actelion.research.chem.conf.BondRotationHelper;
+import com.actelion.research.chem.conf.Conformer;
+import com.actelion.research.chem.conf.TorsionDB;
 import com.actelion.research.chem.forcefield.mmff.ForceFieldMMFF94;
 import com.actelion.research.chem.optimization.Evaluable;
 import com.actelion.research.chem.phesa.AtomicGaussian;
@@ -30,10 +39,11 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 	//private static final double DELTA = -0.01;
 	private static final double LAMBDA = 0.0625;
 	private double e0 = 0.0;
-	private StereoMolecule fitMol;
+	private Conformer fitConf;
+	private StereoMolecule refMol;
 	private PheSAAlignment shapeAlign;
 	private boolean[] isHydrogen;
-	private double[] v; //coordinates of the atoms
+	private double[] v; // internal coordinates of the atoms
 	private double[][] precalcPow;
 	private double[] precalcExp;
     private double oAA;
@@ -41,50 +51,98 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
     private ForceFieldMMFF94 ff;
     private Map<String, Object> ffOptions;
     private double ppWeight;
+	private Coordinates[] origCoords;
+	private Coordinates[] cachedCoords; //used for gradient calculation: ligand coordinates with adjusted dihedral angles, but before rotation and translation
+	private double[][] dRdvi1;
+	private double[][] dRdvi2;
+	private double[][] dRdvi3;
+	private Coordinates origCOM;
+	private BondRotationHelper torsionHelper;
     
-	public EvaluableFlexibleOverlap(PheSAAlignment shapeAlign, StereoMolecule refMol, StereoMolecule fitMol, double ppWeight, boolean[] isHydrogen,double[] v, Map<String, Object> ffOptions) {
+	public EvaluableFlexibleOverlap(PheSAAlignment shapeAlign, StereoMolecule refMol, Conformer fitConf, BondRotationHelper torsionHelper,
+			double ppWeight, boolean[] isHydrogen, Map<String, Object> ffOptions) {
 		ForceFieldMMFF94.initialize(ForceFieldMMFF94.MMFF94SPLUS);
 		this.ffOptions = ffOptions;
-		ff = new ForceFieldMMFF94(fitMol, ForceFieldMMFF94.MMFF94SPLUS, this.ffOptions);
 		this.shapeAlign = shapeAlign;
-		this.fitMol = fitMol;
+		this.fitConf = fitConf;
 		this.isHydrogen = isHydrogen;
 		this.ppWeight = ppWeight;
-		this.v = v;
-		for(int i=0;i<fitMol.getAllAtoms();i++) {
-			v[3*i]=fitMol.getAtomX(i);
-			v[3*i+1]=fitMol.getAtomY(i);
-			v[3*i+2]=fitMol.getAtomZ(i);
-		}
-
+		this.refMol = refMol;
+		this.torsionHelper = torsionHelper;
+		init();
+	}
+	
+	private void init() {
+		ff = new ForceFieldMMFF94(fitConf.getMolecule(), ForceFieldMMFF94.MMFF94SPLUS, this.ffOptions);
 		this.oAA = this.getFGValueShapeSelf(new double[3*refMol.getAllAtoms()], shapeAlign.getRefMolGauss(),true);
 		this.oAApp = this.getFGValueSelfPP(new double[3*refMol.getAllAtoms()], shapeAlign.getRefMolGauss(),true);
+		setInitialState();
+		origCoords = new Coordinates[fitConf.getMolecule().getAllAtoms()];
+		cachedCoords = new Coordinates[fitConf.getMolecule().getAllAtoms()];
+		origCOM = new Coordinates();
+		for(int a=0;a<fitConf.getMolecule().getAllAtoms();a++) {
+			origCoords[a] = new Coordinates(fitConf.getMolecule().getCoordinates(a));
+			cachedCoords[a] = new Coordinates(fitConf.getMolecule().getCoordinates(a));
+			origCOM.add(cachedCoords[a]);
+		}
+		origCOM.scale(1.0/cachedCoords.length);
+		for(Coordinates coords : origCoords) {
+			coords.sub(origCOM);
+		}
+		dRdvi1 = new double[3][3];
+		dRdvi2 = new double[3][3];
+		dRdvi3 = new double[3][3];
+	}
+	
+	public void setInitialState() {
+		int elements = 3+3+torsionHelper.getRotatableBonds().length; //3 translational, 3 rotational, 3 torsion
+		v = new double[elements];
+		v[0] = 0.0;
+		v[1] = 0.0;
+		v[2] = 0.0;
+		Quaternion quat = new Quaternion(1.0,0.0,0.0,0.0);
+		ExponentialMap emap = new ExponentialMap(quat);
+		v[3] = emap.getP().x;
+		v[4] = emap.getP().y;
+		v[5] = emap.getP().z;
+		for(int b=0;b<torsionHelper.getRotatableBonds().length;b++) {
+			int[] atoms = torsionHelper.getTorsionAtoms()[b];
+			v[6+b] = TorsionDB.calculateTorsionExtended(fitConf, atoms);
+		}
+	}
+	
+	private void resetLigCoordinates() {
+		for(int a=0;a<fitConf.getMolecule().getAllAtoms();a++) {
+			fitConf.setX(a, origCoords[a].x);
+			fitConf.setY(a, origCoords[a].y);
+			fitConf.setZ(a, origCoords[a].z);
+		}
 	}
 	
 	public EvaluableFlexibleOverlap(EvaluableFlexibleOverlap e) {
 		this.shapeAlign = e.shapeAlign;
-		this.fitMol = e.fitMol;
+		this.fitConf = e.fitConf;
 		this.isHydrogen = e.isHydrogen;
 		this.v = e.v;	
 		this.precalcPow = e.precalcPow;
 		this.precalcExp = e.precalcExp;
-		this.ff = e.ff;
-
+		init();
 	}
 	
 	@Override
 	public void setState(double[] v){
 		assert this.v.length==v.length;
 		for(int i=0;i<v.length;i++) {
+			if(i>5) { //torsions
+				if(v[i]>Math.PI) {
+					v[i] -= 2*Math.PI;
+				}
+			}
 			this.v[i] = v[i];
+	
 		}
-		ff.setState(v);
-		for(int a=0,i=0;i<fitMol.getAllAtoms();i++) {
-			fitMol.setAtomX(i,v[a++]);
-			fitMol.setAtomY(i,v[a++]);
-			fitMol.setAtomZ(i,v[a++]);
-		}
-		shapeAlign.getMolGauss().update(fitMol);
+		updateLigandCoordinates();
+		shapeAlign.getMolGauss().update(fitConf);
 	}
 	
 	public double[] getState(double[] v){
@@ -109,29 +167,87 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 		return this.shapeAlign;
 	}
 	
-
+	public void updateLigandCoordinates() {
+		resetLigCoordinates();
+		//1. update dihedral angles
+		//2. translate COM of ligand to rotation center
+		//3. apply rotation 
+		//4. translate back
+		//5. apply translation 
+		updateDihedralAngles();
+		for(int a=0;a<fitConf.getMolecule().getAllAtoms();a++) {
+			cachedCoords[a] = new Coordinates(fitConf.getCoordinates(a));
+		}
 	
+		ExponentialMap eMap = new ExponentialMap(v[3],v[4],v[5]);
+		Quaternion q = eMap.toQuaternion();
+		Translation trans = new Translation(origCOM);
+		Rotation rot = new Rotation(q.getRotMatrix().getArray());
+		Translation t = new Translation(v[0],v[1],v[2]);
+		TransformationSequence transformation = new TransformationSequence();
+		transformation.addTransformation(rot);
+		transformation.addTransformation(trans);
+		transformation.addTransformation(t);
+		transformation.apply(fitConf);
+	}
+	
+	private void updateDihedralAngles() {
+		for(int b=0;b<torsionHelper.getRotatableBonds().length;b++) {
+			double targetTorsion = v[6+b];
+			int[] atoms = torsionHelper.getTorsionAtoms()[b];
+			double currentTorsion = TorsionDB.calculateTorsionExtended(fitConf, atoms);
+			double deltaTorsion = targetTorsion - currentTorsion;
+			torsionHelper.rotateAroundBond(b, deltaTorsion,fitConf,false);
+		}
+	}
+	
+
+	/**
+	 * inspired by: 10.1021/acs.jcim.7b00618 (Reflex3D)
+	 * objective function:
+	 * f = -T + lambda *strain^2            
+	 * A is the fixed reference molecule, whereas B is flexible aligned on A3
+	 *                                                             
+	 * where T is the shape Tanimoto : T = O_AB/(O_BB+O_AA-O_AB): with O_AB being the shape overlap between molecules A and B
+	 *                                                             and O_AA and O_BB the self overlaps
+	 * next we calculate the derivate of the shape tanimoto with respect to the cartesian coordinates:
+	 * dT/dx
+	 * we use the product rule: T = u*v
+	 * dT/dx = du/dx*v+u*dv/dx
+	 * u = O_AB and v = (O_BB+O_AA-O_AB)^-1 
+	 * du/dx = dO_AB/dx
+	 * for v we use the chain rule:
+	 * dv/dx = -(dO_BB/dx-dO_AB/dx)*(O_AA+O_BB-O_AB)^-2        
+	 * dT/dx=dO_AB/dx * 1/(O_AA+O_BB-O_AB)-O_AB*(dO_BB/dx-dO_AB/dx)*(O_AA+O_BB-O_AB)^-2 
+	 * and the strain term:
+	 * s = lambda*strain^2 with strain = e-c (e: potential energy, c: energy cutoff)                                                    
+	 * using the chain rule:
+	 * ds/dx = 2*lambda*(e-c)*de/dx
+	 */
 	@Override
 	public double getFGValue(double[] grad) {
+		// first calculate the gradient with respect to the coordinates
+		double[] coordGrad = new double[fitConf.getMolecule().getAllAtoms()*3];
 		double ePot = 0.0;
 		double T = 0.0;
-		double[] overlapGrad = new double[grad.length]; 
-		double[] energyGrad = new double[grad.length]; 
-		double[] selfOverlapGradFit = new double[grad.length];
-		double[] overlapGradPP = new double[grad.length]; 
-		double[] selfOverlapGradFitPP = new double[grad.length];
+		double[] overlapGrad = new double[coordGrad.length]; 
+		double[] energyGrad = new double[coordGrad.length]; 
+		double[] selfOverlapGradFit = new double[coordGrad.length];
+		double[] overlapGradPP = new double[coordGrad.length]; 
+		double[] selfOverlapGradFitPP = new double[coordGrad.length];
 		double oBB = this.getFGValueShapeSelf(selfOverlapGradFit, shapeAlign.getMolGauss(),false);
 		double oAB = this.getFGValueShape(overlapGrad);
 		double oBBpp = this.getFGValueSelfPP(selfOverlapGradFitPP, shapeAlign.getMolGauss(),false);
 		double oABpp = this.getFGValuePP(overlapGradPP);
+		ff.setState(getCartState());
 		ff.addGradient(energyGrad);
 		ePot = ff.getTotalEnergy();
 		double[] dOBB = selfOverlapGradFit;
 		double[] dOAB = overlapGrad;
-		double[] dOBB_dOAB = new double[grad.length];
+		double[] dOBB_dOAB = new double[coordGrad.length];
 		double[] dOBBpp = selfOverlapGradFitPP;
 		double[] dOABpp = overlapGradPP;
-		double[] dOBBpp_dOABpp = new double[grad.length];
+		double[] dOBBpp_dOABpp = new double[coordGrad.length];
 		T = (1.0-ppWeight)*(oAB/(oBB+oAA-oAB))+ppWeight*(oABpp/(oBBpp+oAApp-oABpp));
 		//double value = SCALE*Math.exp(DELTA*(ePot-e0))*T + (ePot-e0);
 		double strainEnergy = ePot-e0;
@@ -151,7 +267,48 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 
 			grad[k] = -dT[k] + strainPrefactor*2*LAMBDA*energyGrad[k];
 		}
+		//to inner coordinates
+		//1. with respect to translational DOG
+		for(int a=0;a<fitConf.getMolecule().getAllAtoms();a++) {
+			grad[0] += coordGrad[3*a]; 
+			grad[1] += coordGrad[3*a+1]; 
+			grad[2] += coordGrad[3*a+2]; 
+		}
+		//2. orientational 
+		//with respect to vector of exponential mapping p
+		// dE/dpj = Tj*vi'*dE/dx
+		//vi': atomic position (after adjustment of torsion values)
+		double[] p = new double[] {v[3],v[4],v[5]};
+		RotationDerivatives transformDerivatives = new RotationDerivatives(p);
+		transformDerivatives.dRdv(0, dRdvi1);
+		transformDerivatives.dRdv(1, dRdvi2);
+		transformDerivatives.dRdv(2, dRdvi3);
+		for(int a=0;a<fitConf.getMolecule().getAllAtoms();a++) {
+			Coordinates vi = cachedCoords[a];
+			Coordinates Tj_vi = vi.rotateC(dRdvi1);
+			grad[3] += coordGrad[3*a]*Tj_vi.x+coordGrad[3*a+1]*Tj_vi.y+coordGrad[3*a+2]*Tj_vi.z;
+			Tj_vi = vi.rotateC(dRdvi2);
+			grad[4] += coordGrad[3*a]*Tj_vi.x+coordGrad[3*a+1]*Tj_vi.y+coordGrad[3*a+2]*Tj_vi.z;
+			Tj_vi = vi.rotateC(dRdvi3);
+			grad[5] += coordGrad[3*a]*Tj_vi.x+coordGrad[3*a+1]*Tj_vi.y+coordGrad[3*a+2]*Tj_vi.z;
+		}
+		//3. torsional gradient
+		for(int b=0;b<torsionHelper.getRotatableBonds().length;b++) {
+			int[] rotatedAtoms = torsionHelper.getSmallerSideAtomLists()[b];
+			int j = torsionHelper.getRotationCenters()[b];
+			int k = torsionHelper.getTorsionAtoms()[b][1] == j ? torsionHelper.getTorsionAtoms()[b][2] : torsionHelper.getTorsionAtoms()[b][1];
+			Coordinates v1 = fitConf.getCoordinates(k).subC(fitConf.getCoordinates(j));
 
+			for(int i : rotatedAtoms) {
+				Coordinates v2 = 
+						fitConf.getCoordinates(i).subC(fitConf.getCoordinates(j));
+				Coordinates dx_dphi = v1.cross(v2);
+				grad[6+b] += dx_dphi.x*coordGrad[3*i] + dx_dphi.y*coordGrad[3*i+1] + 
+						dx_dphi.z*coordGrad[3*i+2];
+			}
+			
+				//state[5+b+1] = TorsionDB.calculateTorsionExtended(ligConf, atoms);
+		}
 
 		return value;
 		
@@ -160,6 +317,36 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 
 	/**
 	 * calculates the gradient of the overlap function with respect to the cartesian coordinates of the atoms
+	 * 
+	 * 
+	 * the overlap between molecules A and B is expressed as a sum over the atom-atom overlaps Vij
+	 *  
+     *                     pi           3/2                 alpha_i*alpha_j*Rij^2
+     * Vij = p_i*p_j*(---------------)        * exp( - -------------------- )      
+     *               alpha_i + alpha_j                     alpha_i + alpha_j 
+     * 
+     *     
+     *              alpha_i*alpha_j*Rij^2
+     *     = a*exp(- --------------------------)
+     *                   alpha_i + alpha_j 
+     * 
+     * Rij^2 = (xi-xj)^2                  
+     * we therefore need the derivative of Vij with respect to the atomic coordinates of atom j (molecule B)
+     * 
+     * we use the chain rule: 
+     * 
+     * d[e^u(x)]/dx = e^u(x)*du(x)/dx
+     * 
+     * with u(x) = -c*Rij^2   and c=(alpha_i*alpha_j)/(alpha_i+alpha_j)
+     * 
+     * u(x) = -c*(xi^2-2xi*xj+xj^2)
+     * 
+     * du(x)/dx=-c*(-2xi+2xj)
+     * 
+     * dVij = -Vij*(alpha_i*alpha_j)/(alpha_i+alpha_j) * (2xj - 2xi)
+     * 
+     * 
+     * 
 	 */
 	
 	public double getFGValueShape(double[] grad) {
@@ -172,10 +359,6 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 			grad[i] = 0;
 		}
 
-		/**
-		 * derivative of ShapeOverlap with respect to the four elements of the quaternion and three elements of translation
-		 * 
-		 */ 
 	    double totalOverlap = 0.0;
 		for(AtomicGaussian refAt:refMolGauss.getAtomicGaussians()){
 			double xi = refAt.getCenter().x;
@@ -184,9 +367,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 			for(AtomicGaussian fitAt:molGauss.getAtomicGaussians()){
 				int a = fitAt.getAtomId();
 				double atomOverlap = 0.0;
-				double xj = v[3*a];
-				double yj = v[3*a+1];
-				double zj = v[3*a+2];
+				double xj = fitConf.getX(a);
+				double yj = fitConf.getY(a);
+				double zj = fitConf.getZ(a);
 				double dx = xi-xj;
 				double dy = yi-yj;
 				double dz = zi-zj;
@@ -203,9 +386,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 					}
 
 				}
-				grad[3*a] += (2*xj-2*xi)*gradientPrefactor;
-				grad[3*a+1] += (2*yj-2*yi)*gradientPrefactor;
-				grad[3*a+2] += (2*zj-2*zi)*gradientPrefactor;
+				grad[3*a] += (xj-xi)*gradientPrefactor;
+				grad[3*a+1] += (yj-yi)*gradientPrefactor;
+				grad[3*a+2] += (zj-zi)*gradientPrefactor;
 				}
 
 		
@@ -237,9 +420,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 					}
 
 				}
-				grad[3*a] += (2*xj-2*xi)*gradientPrefactor;
-				grad[3*a+1] += (2*yj-2*yi)*gradientPrefactor;
-				grad[3*a+2] += (2*zj-2*zi)*gradientPrefactor;
+				grad[3*a] += (xj-xi)*gradientPrefactor;
+				grad[3*a+1] += (yj-yi)*gradientPrefactor;
+				grad[3*a+2] += (zj-zi)*gradientPrefactor;
 				}
 
 		
@@ -259,7 +442,7 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 		for(int i=0;i<grad.length;i++) {
 			grad[i] = 0;
 		}
-
+		double[] coords = getCartState();
 		/**
 		 * derivative of ShapeOverlap with respect to the four elements of the quaternion and three elements of translation
 		 * 
@@ -272,9 +455,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 			for(PPGaussian fitPP:molGauss.getPPGaussians()){
 				int a = fitPP.getAtomId();
 				double atomOverlap = 0.0;
-				double xj = v[3*a];
-				double yj = v[3*a+1];
-				double zj = v[3*a+2];
+				double xj = fitConf.getX(a);
+				double yj = fitConf.getY(a);
+				double zj = fitConf.getZ(a);
 				double dx = xi-xj;
 				double dy = yi-yj;
 				double dz = zi-zj;
@@ -290,10 +473,10 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 						atomOverlap *= sim;
 						totalOverlap += atomOverlap;
 						gradientPrefactor = atomOverlap*-2*refPP.getWidth()*fitPP.getWidth()/(refPP.getWidth()+fitPP.getWidth());
-						grad[3*a] += (2*xj-2*xi)*gradientPrefactor*sim;
-						grad[3*a+1] += (2*yj-2*yi)*gradientPrefactor*sim;
-						grad[3*a+2] += (2*zj-2*zi)*gradientPrefactor*sim;
-						fitPP.getPharmacophorePoint().getDirectionalityDerivativeCartesian(grad, v, fitPP.getPharmacophorePoint().getDirectionality(), sim);					}
+						grad[3*a] += (xj-xi)*gradientPrefactor*sim;
+						grad[3*a+1] += (yj-yi)*gradientPrefactor*sim;
+						grad[3*a+2] += (zj-zi)*gradientPrefactor*sim;
+						fitPP.getPharmacophorePoint().getDirectionalityDerivativeCartesian(grad, coords, fitPP.getPharmacophorePoint().getDirectionality(), sim);					}
 
 				}
 
@@ -351,9 +534,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 			zi = refAt.getCenter().z;
 		}
 		else {
-			xi = v[3*a];
-			yi = v[3*a+1];
-			zi = v[3*a+2];
+			xi = refMol.getAtomX(a);
+			yi = refMol.getAtomY(a);
+			zi = refMol.getAtomZ(a);
 		}
 
 		if(rigid) {
@@ -362,9 +545,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 			zj = fitAt.getCenter().z;
 		}
 		else {
-			xj = v[3*b];
-			yj = v[3*b+1];
-			zj = v[3*b+2];
+			xj = fitConf.getX(b);
+			yj = fitConf.getY(b);
+			zj = fitConf.getZ(b);
 		}
 		double dx = xi-xj;
 		double dy = yi-yj;
@@ -403,7 +586,7 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 		for(int i=0;i<grad.length;i++) {
 			grad[i] = 0;
 		}
-
+		double[] coord = getCartState();
 		/**
 		 * derivative of ShapeOverlap with respect to Cartesian coordinates
 		 */ 
@@ -416,9 +599,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 				zi = refPP.getCenter().z;
 			}
 			else {
-				xi = v[3*a];
-				yi = v[3*a+1];
-				zi = v[3*a+2];
+				xi = fitConf.getX(a);
+				yi = fitConf.getY(a);
+				zi = fitConf.getZ(a);
 			}
 			for(PPGaussian fitPP:molVol.getPPGaussians()){
 				int b = fitPP.getAtomId();
@@ -430,9 +613,9 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 					zj = fitPP.getCenter().z;
 				}
 				else {
-					xj = v[3*b];
-					yj = v[3*b+1];
-					zj = v[3*b+2];
+					xj = fitConf.getX(b);
+					yj = fitConf.getY(b);
+					zj = fitConf.getZ(b);
 				}
 				double dx = xi-xj;
 				double dy = yi-yj;
@@ -454,22 +637,30 @@ public class EvaluableFlexibleOverlap implements Evaluable  {
 							grad[3*a] += (2*xj-2*xi)*gradientPrefactor*sim;
 							grad[3*a+1] += (2*yj-2*yi)*gradientPrefactor*sim;
 							grad[3*a+2] += (2*zj-2*zi)*gradientPrefactor*sim;
-							fitPP.getPharmacophorePoint().getDirectionalityDerivativeCartesian(grad, v, fitPP.getPharmacophorePoint().getDirectionality(), sim);					}
+							fitPP.getPharmacophorePoint().getDirectionalityDerivativeCartesian(grad, coord, fitPP.getPharmacophorePoint().getDirectionality(), sim);					}
 					}
 				}
-
 				}
+		}
+		return totalOverlap; 
+	}
+	
+	public double[] getCartState(){
+		double[] cartState = new double[3*fitConf.getMolecule().getAllAtoms()];
+		for(int a=0;a<fitConf.getMolecule().getAllAtoms();a++) {
+			cartState[3*a] = fitConf.getCoordinates(a).x;
+			cartState[3*a+1] = fitConf.getCoordinates(a).y;
+			cartState[3*a+2] = fitConf.getCoordinates(a).z;
 			
 		}
-
-
-		return totalOverlap; 
-		
+		return cartState;
+	}
 	
+	public Conformer getFitConf() {
+		return fitConf;
 	}
 	
 	
-
 	@Override
 	public EvaluableFlexibleOverlap clone() {
 		return new EvaluableFlexibleOverlap(this);
